@@ -1,7 +1,9 @@
-/* Copyright (c) 1987, 1988  Stanley T. Shebs. */
+/* Copyright (c) 1987, 1988  Stanley T. Shebs, University of Utah. */
 /* This program may be used, copied, modified, and redistributed freely */
 /* for noncommercial purposes, so long as this notice remains intact. */
 
+/* RCS $Header: /home/users/brossard/X/commands/x11conq/RCS/dir.h,v 2.0 90/10/20 12:51:10 brossard Exp $ */
+
 /* Definitions for directions of the compass. */
 
 /* The terrain model is based on hexes arranged in horizontal rows.  This */
@@ -27,11 +29,12 @@
 
 #define DIRCHARS "ulnbhy"
 
-#define random_dir() (lrand48() % NUMDIRS)
+#define random_dir() (random(6))
 
-#define for_all_directions(dir)  for (dir = 0; dir < NUMDIRS; ++dir)
+#define for_all_directions(dir)  for (dir = 0; dir < 6; ++dir)
 
-#define normalize_dir(d)	(((d)+NUMDIRS)%NUMDIRS)
-#define opposite_dir(d) (((d) + 3) % NUMDIRS)
-#define	right_of(d)	(((d) + 1) % NUMDIRS)
-#define	left_of(d)	(((d) + NUMDIRS - 1) % NUMDIRS)
+#define opposite_dir(d) (((d) + 3) % 6)
+
+extern char *dirnames[];
+
+extern int dirx[], diry[];
